var _ = require('underscore')._
  , util = require('util')
  , events = require('events')
  , request = require('http').request
  , Klient

module.exports = Klient = function(options) {
  this.defaults = { host: 'localhost', port: 8081 }
  this.options = _.extend({}, this.defaults, options)
  this.server = this.options.server
  this.deferred = []
  this.started = false
  this.pending = false

  events.EventEmitter.call(this)
}

util.inherits(Klient, events.EventEmitter)

Klient.prototype.processDeferred = function() {
  var self = this
    , deferred = self.deferred

  process.nextTick(function() {
    _(deferred.length).times(function() {
      self.request(deferred.shift())
    })
  })
}

Klient.prototype.startServer = function() {
  var self = this
    , server = self.server
    , host = self.options.host
    , port = self.options.port
    , callback

  callback = function() {
    self.pending = false
    self.started = true
    self.processDeferred()
  }

  self.pending = true

  server.listen(port, host, callback)
}

Klient.prototype.request = function(options) {
  var self = this
    , server = self.server
    , data, req

  if(!server) {
    return process.nextTick(function() {
      self.emit('error', new Error('Server is not defined'))
    })
  }

  if(!server.fd || !self.started) {
    self.deferred.push(options)

    if(!self.started && !self.pending) {
      self.startServer()
    }

    return
  }

  options = _.extend({}, self.options, options)
  req = request(options)

  process.nextTick(function() {
    req.on('response', function(res) {
      var body = ''

      res.setEncoding('utf8')
      res.on('data', function(chunk) {
        body += chunk
      })
      res.on('end', function() {
        if(res.headers['content-type'] === 'application/json') {
          body = JSON.parse(body)
        }

        res.body = body

        self.emit('success', res)
      })
    }).on('error', function(err) {
      self.emit('error', err)
    })

    if(options.data) {
      req.end(options.data)
    } else {
      req.end()
    }
  })
}

Klient.prototype.promisedRequest = function(options) {
  var self = this
    , promise = new(events.EventEmitter)

  self.on('success', function(res) {
    promise.emit('success', res)
  })
  self.on('error', function(err) {
    promise.emit('error', err)
  })

  self.request(options)

  return promise
}
Klient.prototype.get = function(options) {
  _.extend(options, { method: 'get' })

  return this.promisedRequest(options)
}
Klient.prototype.post = function(options) {
  _.extend(options, { method: 'post' })

  return this.promisedRequest(options)
}
Klient.prototype.put = function(options) {
  _.extend(options, { method: 'put' })

  return this.promisedRequest(options)
}
Klient.prototype.del = function(options) {
  _.extend(options, { method: 'delete' })

  return this.promisedRequest(options)
}
